#!/usr/bin/env python
# vim: sts=4 sw=4 et
#    This is a component of EMC
#    copyright 2016 Andy Pugh

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA''''''

import os,sys
from gladevcp.persistence import IniFile,widget_defaults,set_debug,select_widgets
import hal
import hal_glib
import gtk
import glib
import linuxcnc
import csv

debug = 0

status = ["Idle","Backstop", "NA", "NA", "NA", "Lowering", "Bending", "Tweak?", "Return", "Overload",
          "HomeB", "HomeX1", "HomeX2", "Home Z", "NA", "NA", "NA", "NA", "NA", "NA",
          "JogB", "JogX1", "JogX2", "JogZ", "NA", "NA","NA", "NA","NA", "NA"]

class HandlerClass:
    row = -1
    old_status = 0
    filename = ""
    
    def on_destroy(self,obj,data=None):
        print("destroy event")
        self.ini.save_state(self)
    
    def on_max_pressure_change(self, obj, Data=None):
        gauge = self.builder.get_object("pressure-gauge")
        gauge.props.z0_border = obj.get_value_as_int() * 0.9
        gauge.props.z1_border = obj.get_value_as_int()
        gauge.props.max = obj.get_value_as_int() * 1.1

    def on_status_change(self, hal_pin, data=None):
        s = int(hal_pin.get())
        self.builder.get_object('label_status').set_text(status[s])
        if s == 0 and self.old_status > 0:
            if self.old_status in [5, 6, 7, 8]:
                self.row += 1
                if self.row >= len(self.liststore):
                    self.row = 0
                self.builder.get_object('treeview1').set_cursor(self.row)
                self.halcomp['start'] = self.liststore[self.row][0]
                self.halcomp['end'] = self.liststore[self.row][1]
                self.halcomp['backstop'] = self.liststore[self.row][2]
                self.halcomp['backstop-r'] = self.liststore[self.row][3]         
            elif self.old_status in [10,11,12,13,14]:
                print("homed")
                self.halcomp['home-select'] = 0
                
        if s == 0:
            self.builder.get_object('jogging').set_sensitive(True)
            self.builder.get_object('homing').set_sensitive(True)
        elif s in [10, 11, 12, 13]:
            self.builder.get_object('jogging').set_sensitive(False)
            self.builder.get_object('homing').set_sensitive(True)
        elif s in [20, 21, 22, 23]:
            self.builder.get_object('jogging').set_sensitive(True)
            self.builder.get_object('homing').set_sensitive(False)
        else:
            self.builder.get_object('jogging').set_sensitive(False)
            self.builder.get_object('homing').set_sensitive(False)
        self.old_status = s
        
    def on_estop_change(self, hal_pin, data=None):
        e = hal_pin.get()
        if e:
            print("enable")
            self.builder.get_object('pump').set_active(1)
            self.builder.get_object('enable').set_active(1)
        else:
            print("unenable")
            self.builder.get_object('pump').set_active(0)
            self.builder.get_object('enable').set_active(0)

    def __init__(self, halcomp,builder,useropts):
        self.halcomp = halcomp
        self.builder = builder
        self.ini_filename = 'pressgui.ini'
        self.defaults = {  IniFile.vars: {'axis_mask' : 7, 'bend_max' : 1, 'down_max': 5,
                          'max0':6, 'min0':0, 'max1':6, 'min1':0, 'max2':6, 'min2':0, 'max3':100, 'min3':0 },
                           IniFile.widgets : widget_defaults(select_widgets(self.builder.get_objects(), hal_only=False,output_only = True))
                        }
        self.ini = IniFile(self.ini_filename,self.defaults,self.builder)
        self.ini.restore_state(self)
        
        self.builder.get_object('home1').set_visible(self.axis_mask & 2)
        self.builder.get_object('jog1').set_visible(self.axis_mask & 2)
        self.builder.get_object('home2').set_visible(self.axis_mask & 4)
        self.builder.get_object('jog2').set_visible(self.axis_mask & 4)
        self.builder.get_object('home3').set_visible(self.axis_mask & 8)
        self.builder.get_object('jog3').set_visible(self.axis_mask & 8)
        self.builder.get_object('adj_downspeed').set_upper(self.down_max)
        self.builder.get_object('adj_bendspeed').set_upper(self.bend_max)
        
        self.liststore = self.builder.get_object('liststore1')
        self.cmd = linuxcnc.command()
        
        # simple pins
        halcomp.newpin("start", hal.HAL_FLOAT, hal.HAL_OUT)
        halcomp.newpin("end", hal.HAL_FLOAT, hal.HAL_OUT)
        halcomp.newpin("backstop", hal.HAL_FLOAT, hal.HAL_OUT)
        halcomp.newpin("backstop-r", hal.HAL_FLOAT, hal.HAL_OUT)
        halcomp.newpin("jog-select", hal.HAL_S32, hal.HAL_OUT)
        halcomp["jog-select"] = -1
        halcomp.newpin("home-select", hal.HAL_S32, hal.HAL_OUT)
        halcomp["home-select"] = -1
        halcomp.newpin("jog-increment", hal.HAL_FLOAT, hal.HAL_OUT)
        halcomp["jog-increment"] = 0.00001
        
        # Pins with callbacks
        self.status = hal_glib.GPin(halcomp.newpin('status',  hal.HAL_S32, hal.HAL_IN))
        self.status.connect('value-changed', self.on_status_change)
        self.estop = hal_glib.GPin(halcomp.newpin('estop-in',  hal.HAL_BIT, hal.HAL_IN))
        self.status.connect('value-changed', self.on_estop_change)

    def handle_home(self, obj, Data=None):
        if obj.name == "home0":
            self.halcomp['home-select'] = 0
        if obj.name == "home1":
            self.halcomp['home-select'] = 1
        if obj.name == "home2":
            self.halcomp['home-select'] = 2
        if obj.name == "home3":
            self.halcomp['home-select'] = 3
            
    def handle_jog(self, obj, Data=None):
        if obj.get_active() == False:
            self.halcomp['jog-select'] = -1
        else:
            for o in obj.get_parent().get_children():
                if o.name != obj.name:
                    o.set_active(False)
            if obj.name == "jog0": 
                self.halcomp['jog-select'] = 0
            if obj.name == "jog1": 
                self.halcomp['jog-select'] = 1
            if obj.name == "jog2": 
                self.halcomp['jog-select'] = 2
            if obj.name == "jog3": 
                self.halcomp['jog-select'] = 3 
                
    def increments(self, obj, Data=None):
            for o in obj.get_parent().get_children():
                o.handler_block_by_func(self.increments)
                if o != obj:
                    o.set_active(False)
                else:
                    o.set_active(True)
                o.handler_unblock_by_func(self.increments)
            if obj.name == "inc0": 
                self.halcomp['jog-increment'] = 0.0001
            if obj.name == "inc1": 
                self.halcomp['jog-increment'] = 0.00001
            if obj.name == "inc2": 
                self.halcomp['jog-increment'] = 0.000001
            if obj.name == "inc3": 
                self.halcomp['jog-increment'] = 0.0000001
            
    def cell_edit(self, row, col, val):
        self.liststore[row][col] = val
        
    def row_click(self, obj, Data=None):
        print(obj)



    def edit1(self, obj, row, val):
        try:
            f = float(val)
            if f > 2000 or f < 0:
                self.cell_edit(row, 0, '')
                return
            self.cell_edit(row, 0, '%0.3f'%f)
        except:
            self.cell_edit(row, 0, '')
            return
    def edit2(self, obj, row, val):
        try:
            f = float(val)
            if f > 180 or f < -180:
                self.cell_edit(row, 1, '')
                return
            self.cell_edit(row, 1, '%0.3f'%f)
        except:
            self.cell_edit(row, 1, '')
            return
    def edit3(self, obj, row, val):
        try:
            f = float(val)
            if f > 360 or f < -270:
                self.cell_edit(row, 2, '')
                return
            self.cell_edit(row, 2, '%0.3f'%f)
        except:
            self.cell_edit(row, 2, '')
            return  
    def edit4(self, obj, row, val):
        try:
            f = float(val)
            if f > 360 or f < -270:
                self.cell_edit(row, 3, '')
                return
            self.cell_edit(row, 3, '%0.3f'%f)
        except:
            self.cell_edit(row, 3, '')
            return  
    def edit5(self, obj, row, val):
        print(val)
        self.cell_edit(row, 4, val)

    def add_row(self, obj, Data=None):
        self.liststore.append()
        
    def delete_row(self, obj, Data=None):
        selection = self.builder.get_object('treeview1').get_selection()
        model, paths = selection.get_selected_rows()
        for path in paths:
            iter = model.get_iter(path)
            model.remove(iter)

    def to_start(self, obj, Data=None):
        global row
        row = 0
        self.builder.get_object('treeview1').set_cursor(row)

    def load_file(self, obj, Data=None):
        dlg = gtk.FileChooserDialog("Open..", None, gtk.FILE_CHOOSER_ACTION_OPEN,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        response = dlg.run()
        self.filename = dlg.get_filename()
        dlg.destroy()
        if response != gtk.RESPONSE_OK:
            return
        self.liststore.clear()
        with open(self.filename) as myfile:
            csvread = csv.reader(myfile, delimiter=',')
            for row in csvread:
                self.liststore.append(row)
                
    def save_csv(self, filename):
        if filename == "":
            dlg = gtk.FileChooserDialog("Save..", None, gtk.FILE_CHOOSER_ACTION_SAVE,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK))
            response = dlg.run()
            self.filename = dlg.get_filename()
            dlg.destroy()
            if response != gtk.RESPONSE_OK:
                return
        with open(self.filename, "w") as myfile:
            csvwriter = csv.writer(myfile, delimiter=',')
            for row in self.liststore:
                csvwriter.writerow(row)
    
    def save_file(self, obj, Data=None):
        if self.filename == "":
            self.save_csv("")
        else:
            self.save_csv(self.filename)

    def saveas_file(self, obj, Data=None):
        self.save_csv("")
        
    def file_new(self, obj, Data=None):
        self.liststore.clear()
        
    def quit(self, obj, Data=None):
        gtk.main_quit()
        
def get_handlers(halcomp,builder,useropts):

    global debug
    for cmd in useropts:
        exec cmd in globals()

    set_debug(debug)
    
    return [HandlerClass(halcomp,builder,useropts)]



